﻿
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;

/* Root of a polynomialby Newton Maehly                 */
/* x^5 - 5*x^3 + 4*x + 1                                */
/*                                                      */


namespace WindowsFormsApplication1
{

    public partial class MainWin : Form
    {
        public const int order = 6; // number of reference points
        public const double startingpoint = 5.0; // number of reference points
        public const int datapoints = 1000; // number of datapoints for the interpolation
        public const double deltaX = 0.5; // sample time 

        public TextBox[,] tbm;
        public TextBox[] tbs;
        public TextBox[] tbx;

       
        double[] yp = new double[datapoints];
        double[] xp = new double[datapoints];
        double[] y_k = new double[order];
        double[] x = new double[order];
        bool[] valid = new bool[order];
        double[] a = new double[order];
        double xk_max, xk_min;
       
        
        public MainWin()
        {
            int i;
            tbs = new TextBox[10];
            tbx = new TextBox[10];

            InitializeComponent();

            tbs[0] = textBoxY1; tbs[1] = textBoxY2; tbs[2] = textBoxY3; tbs[3] = textBoxY4; tbs[4] = textBoxY5;
            tbs[5] = textBoxY6; tbs[6] = textBoxY7; tbs[7] = textBoxY8; tbs[8] = textBoxY9; tbs[9] = textBoxY10;

            tbx[0] = textBoxX1; tbx[1] = textBoxX2; tbx[2] = textBoxX3; tbx[3] = textBoxX4; tbx[4] = textBoxX5;
            tbx[5] = textBoxX6; tbx[6] = textBoxX7; tbx[7] = textBoxX8; tbx[8] = textBoxX9; tbx[9] = textBoxX10;
 
            // some sample values for a
            a[0] = 1.0;
            a[1] = 0.0;
            a[2] = -5.0;
            a[3] = 0.0;
            a[4] = 4.0;
            a[5] = 1.0;


            if (order < 10)
            {
                for (i = 0; i < order; i++)
                {
                    tbs[i].Text = Convert.ToString(a[i]);
                }
                for (i = order; i < 10; i++)
                {
                    tbs[i].Visible = false;
                    tbx[i].Visible = false;
                }
            }
        }

        public void Interpolate(int maxPoints, double minX, double maxX)
        {
          int i, k;
            for (i = 0; i < maxPoints; i++)
            {
                yp[i] = 0;
                xp[i] = ((double)(i) * (maxX - minX) / (double)maxPoints + minX);
                for (k = 0; k < order; k++)
                {
                    yp[i] = yp[i] + a[k] * Math.Pow(xp[i], (double)(order - k - 1));
                }
            }
        }

        public void DrawGraph(int maxPoints, double minX, double maxX, bool doClear)
        {
            Point p1, p2;
            int j;
            double maxTime = -1.0;
            double minTime = 1.0;
            double maxVal = -1.0;
            double minVal = 1.0;
            double scalefactor;
            bool drawCeroline;
            for (j = 0; j < maxPoints - 1; j++)
            {
                if ((xp[j] >= minX) && (xp[j] <= maxX))
                {
                    if (maxTime < xp[j])
                        maxTime = xp[j];
                    if (minTime > xp[j])
                        minTime = xp[j];
                    if (maxVal < yp[j])
                        maxVal = yp[j];
                    if (minVal > yp[j])
                        minVal = yp[j];
                }
            }
            maxTime = maxTime * 1.05;
            maxTime = maxTime * 1.05;
            if (minVal < 0)
            {
                if (maxVal > 0)
                {
                    if (maxVal > Math.Abs(minVal))
                        scalefactor = pGraph.Height / maxVal / 2.2;
                    else
                        scalefactor = pGraph.Height / Math.Abs(minVal) / 2.2;
                }
                else
                    scalefactor = 5.0;
                drawCeroline = true;
            }
            else
            {
                if (maxVal > 0)
                    scalefactor = pGraph.Height / (maxVal) / 1.1;
                else
                    scalefactor = 1.0;
                drawCeroline = false;
            }
            p1 = new Point();
            p2 = new Point();
            Graphics g = pGraph.CreateGraphics();
            if (doClear)
                g.Clear(Color.White);
            Pen bluePen = new Pen(Color.Blue, 2);
            Pen redPen = new Pen(Color.Red, 2);
            Pen blackPen = new Pen(Color.Black, 2);
            if (drawCeroline)
            {
                p1.X = 0;
                p1.Y = (pGraph.Height / 2);
                p2.X = pGraph.Width;
                p2.Y = (pGraph.Height / 2);
                g.DrawLine(blackPen, p1, p2);
            }
            bluePen.Width = 1;
            if (maxTime > 0) // draw interpolated graph
            {
                for (j = 0; j < maxPoints - 1; j++)
                {
                    if ((xp[j] >= minX) && (xp[j] <= maxX))
                    {
                        p1.X = Convert.ToInt32(((xp[j] - (double)(minTime)) / (double)(maxTime - minTime)) * pGraph.Width);
                        p2.X = Convert.ToInt32(((xp[j + 1] - (double)(minTime)) / (double)(maxTime - minTime)) * pGraph.Width);
                        if (drawCeroline)
                        {
                            p1.Y = (pGraph.Height / 2) - Convert.ToInt32(Math.Round(yp[j] * scalefactor));
                            p2.Y = (pGraph.Height / 2) - Convert.ToInt32(Math.Round(yp[j + 1] * scalefactor));
                        }
                        else
                        {
                            p1.Y = (pGraph.Height) - Convert.ToInt32(Math.Round(yp[j] * scalefactor));
                            p2.Y = (pGraph.Height) - Convert.ToInt32(Math.Round(yp[j + 1] * scalefactor));
                        }
                        if (p2.X > p1.X)
                            g.DrawLine(bluePen, p1, p2);
                    }
                }
                for (j = 0; j < order; j++) // mark reference points
                {
                    float emSize = 10;
                    if (valid[j])
                    {
                        p1.X = Convert.ToInt32((x[j] - (double)(minTime)) * pGraph.Width / (double)(maxTime - minTime));
                        if (drawCeroline)
                            p1.Y = (pGraph.Height / 2) - Convert.ToInt32(Math.Round(y_k[j] * scalefactor));
                        else
                            p1.Y = (pGraph.Height) - Convert.ToInt32(Math.Round(y_k[j] * scalefactor));
                        g.DrawEllipse(redPen, (int)(p1.X - 1.0), (int)(p1.Y - 1.0), 2, 2);
                        g.DrawString(Convert.ToString(Math.Round(x[j], 2)), new Font(FontFamily.GenericSansSerif, emSize, FontStyle.Regular),
                                      new SolidBrush(Color.Black), p1.X, p1.Y + 10);
                    }
                }
            }
            g.Dispose();
        }


        private void button1_Click(object sender, EventArgs e)
        {
            double bn;
            double cn;
            double xk, xk_old, dx, dx_old;
            double xz;
            int i, j, n;
            n = 0;
            xk_max = -1.0E18;
            xk_min = 1.0E18;
            xk_old = startingpoint;
            xk = xk_old;
            while (n < order-1)
            {
                if (n == 0)
                {
                    xk = xk_old;
                }
                else
                {
                    xk = (xk + xk_old) / 2.0;
                    xk_old = xk;
                }
                xz = 0;
                dx = xk;
                dx_old = 1E8;
                while ((Math.Abs(dx) > 1E-8) && (Math.Abs(xk) < 1E8)) // check if we are close enough
                {
                    dx_old = dx;
                    bn = a[0];
                    cn = bn;
                    for (i = 1; i < order; i++)
                    {
                        bn = xk * bn + a[i];
                        if (i < order - 1)
                            cn = xk * cn + bn;
                    }
                    if (n == 0)  
                    {
                        if (cn != 0) // no root found jet => go the the next P
                        {
                            dx = bn / cn;
                            xk = xk - dx;
                        }
                        else
                            if(Math.Abs(bn) < 1E-8) // that's allready a root point
                                dx = 0.0;
                    }
                    else
                    {
                        if (Math.Abs(bn) > 1E-18)  // one root found => keep it and subtract it from the polynomial
                        {
                            xz = 0;
                            for (j = 0; j < n; j++)
                                xz = xz + 1.0 / (xk - x[j]);
                            dx = 1.0 / (cn / bn - xz);
                            xk = xk - dx;
                        }
                        else
                            xk = xk + 0.5; // we started right at one root point => just move a bit :-)
                    }
                }
                if (Math.Abs(xk) < 1E8)
                {
                    if (Math.Abs(dx) <= Math.Abs(dx_old))  // register root in the list
                    {
                        x[n] = xk;
                        valid[n] = (Math.Abs(dx) != 0) || (n == 0); //(a[order - 1] != 0);
                        if (xk > xk_max)
                            xk_max = xk;
                        if (xk < xk_min)
                            xk_min = xk;
                    }
                    else
                    {
                        x[n] = 0;
                        valid[n] = false;
                    }
                    n++;
                }
                else
                    n = order;
            }

            if (xk_max == -1.0E18)
                xk_max = 0;
            if (xk_min == 1.0E18)
                xk_min = xk_max - 0.1;
            
            Interpolate(datapoints, xk_min - 0.5, xk_max + 0.5);
            DrawGraph(datapoints, xk_min - 0.5, xk_max + 0.5, true);
            if (order < 10)
            {
                for (i = 0; i < order; i++)
                {
                    tbx[i].Text = Convert.ToString(x[i]);
                }
            }
        }

        private void pGraph_Paint(object sender, PaintEventArgs e)
        {
            DrawGraph(datapoints, xk_min - 0.5, xk_max + 0.5, false);
        }  
    }
}
